@@ -16,24 +16,25 @@ import (
 	"github.com/wader/fq/pkg/scalar"
 )
 
-var probeFormat decode.Group
+var probeGroup decode.Group
 
 func init() {
-	interp.RegisterFormat(decode.Format{
-		Name:        format.GZIP,
-		Description: "gzip compression",
-		Groups:      []string{format.PROBE},
-		DecodeFn:    gzDecode,
-		Dependencies: []decode.Dependency{
-			{Names: []string{format.PROBE}, Group: &probeFormat},
-		},
-	})
+	interp.RegisterFormat(
+		format.Gzip,
+		&decode.Format{
+			Description: "gzip compression",
+			Groups:      []*decode.Group{format.Probe},
+			DecodeFn:    gzDecode,
+			Dependencies: []decode.Dependency{
+				{Groups: []*decode.Group{format.Probe}, Out: &probeGroup},
+			},
+		})
 }
 
-const delfateMethod = 8
+const deflateMethod = 8
 
 var compressionMethodNames = scalar.UintMapSymStr{
-	delfateMethod: "deflate",
+	deflateMethod: "deflate",
 }
 
 var osNames = scalar.UintMapSymStr{
@@ -77,7 +78,7 @@ func gzDecode(d *decode.D) any {
 	})
 	d.FieldU32("mtime", scalar.UintActualUnixTime(time.RFC3339))
 	switch compressionMethod {
-	case delfateMethod:
+	case deflateMethod:
 		d.FieldU8("extra_flags", deflateExtraFlagsNames)
 	default:
 		d.FieldU8("extra_flags")
@@ -101,14 +102,15 @@ func gzDecode(d *decode.D) any {
 
 	var rFn func(r io.Reader) io.Reader
 	switch compressionMethod {
-	case delfateMethod:
+	case deflateMethod:
 		// bitio.NewIOReadSeeker implements io.ByteReader so that deflate don't do own
 		// buffering and might read more than needed messing up knowing compressed size
 		rFn = func(r io.Reader) io.Reader { return flate.NewReader(r) }
 	}
 
 	if rFn != nil {
-		readCompressedSize, uncompressedBR, dv, _, _ := d.TryFieldReaderRangeFormat("uncompressed", d.Pos(), d.BitsLeft(), rFn, probeFormat, nil)
+		readCompressedSize, uncompressedBR, dv, _, _ :=
+			d.TryFieldReaderRangeFormat("uncompressed", d.Pos(), d.BitsLeft(), rFn, &probeGroup, format.Probe_In{})
 		if uncompressedBR != nil {
 			if dv == nil {
 				d.FieldRootBitBuf("uncompressed", uncompressedBR)
