@@ -205,7 +205,7 @@ func decodeLang(d *decode.D) string {
 }
 
 // Quicktime time seconds in January 1, 1904 UTC
-var quicktimeEpochDate = time.Date(1904, time.January, 4, 0, 0, 0, 0, time.UTC)
+var quicktimeEpochDate = time.Date(1904, time.January, 1, 0, 0, 0, 0, time.UTC)
 
 var uintActualQuicktimeEpoch = scalar.UintActualDate(quicktimeEpochDate, time.RFC3339)
 
@@ -676,48 +676,48 @@ func decodeBox(ctx *decodeContext, d *decode.D, typ string) {
 			i++
 		})
 	case "avcC":
-		_, v := d.FieldFormat("descriptor", avcDCRFormat, nil)
-		avcDcrOut, ok := v.(format.AvcDcrOut)
+		_, v := d.FieldFormat("descriptor", &avcDCRGroup, nil)
+		avcDcrOut, ok := v.(format.AVC_DCR_Out)
 		if !ok {
 			panic(fmt.Sprintf("expected AvcDcrOut got %#+v", v))
 		}
 		if t := ctx.currentTrack(); t != nil {
-			t.formatInArg = format.AvcAuIn{LengthSize: avcDcrOut.LengthSize} //nolint:gosimple
+			t.formatInArg = format.AVC_AU_In{LengthSize: avcDcrOut.LengthSize} //nolint:gosimple
 		}
 	case "hvcC":
-		_, v := d.FieldFormat("descriptor", hevcCDCRFormat, nil)
-		hevcDcrOut, ok := v.(format.HevcDcrOut)
+		_, v := d.FieldFormat("descriptor", &hevcCDCRGroup, nil)
+		hevcDcrOut, ok := v.(format.HEVC_DCR_Out)
 		if !ok {
 			panic(fmt.Sprintf("expected HevcDcrOut got %#+v", v))
 		}
 		if t := ctx.currentTrack(); t != nil {
-			t.formatInArg = format.HevcAuIn{LengthSize: hevcDcrOut.LengthSize} //nolint:gosimple
+			t.formatInArg = format.HEVC_AU_In{LengthSize: hevcDcrOut.LengthSize} //nolint:gosimple
 		}
 	case "dfLa":
 		d.FieldU8("version")
 		d.FieldU24("flags")
-		_, v := d.FieldFormat("descriptor", flacMetadatablocksFormat, nil)
-		flacMetadatablockOut, ok := v.(format.FlacMetadatablocksOut)
+		_, v := d.FieldFormat("descriptor", &flacMetadatablocksGroup, nil)
+		flacMetadatablockOut, ok := v.(format.FLAC_Metadatablocks_Out)
 		if !ok {
 			panic(fmt.Sprintf("expected FlacMetadatablockOut got %#+v", v))
 		}
 		if flacMetadatablockOut.HasStreamInfo {
 			if t := ctx.currentTrack(); t != nil {
-				t.formatInArg = format.FlacFrameIn{BitsPerSample: int(flacMetadatablockOut.StreamInfo.BitsPerSample)}
+				t.formatInArg = format.FLAC_Frame_In{BitsPerSample: int(flacMetadatablockOut.StreamInfo.BitsPerSample)}
 			}
 		}
 	case "dOps":
-		d.FieldFormat("descriptor", opusPacketFrameFormat, nil)
+		d.FieldFormat("descriptor", &opusPacketFrameGroup, nil)
 	case "av1C":
-		d.FieldFormat("descriptor", av1CCRFormat, nil)
+		d.FieldFormat("descriptor", &av1CCRGroup, nil)
 	case "vpcC":
 		d.FieldU8("version")
 		d.FieldU24("flags")
-		d.FieldFormat("descriptor", vpxCCRFormat, nil)
+		d.FieldFormat("descriptor", &vpxCCRGroup, nil)
 	case "esds":
 		d.FieldU32("version")
-		_, v := d.FieldFormat("descriptor", mpegESFormat, nil)
-		mpegEsOut, ok := v.(format.MpegEsOut)
+		_, v := d.FieldFormat("descriptor", &mpegESGroup, nil)
+		mpegEsOut, ok := v.(format.MPEG_ES_Out)
 		if !ok {
 			panic(fmt.Sprintf("expected mpegEsOut got %#+v", v))
 		}
@@ -725,7 +725,7 @@ func decodeBox(ctx *decodeContext, d *decode.D, typ string) {
 		if t := ctx.currentTrack(); t != nil && len(mpegEsOut.DecoderConfigs) > 0 {
 			dc := mpegEsOut.DecoderConfigs[0]
 			t.objectType = dc.ObjectType
-			t.formatInArg = format.AACFrameIn{ObjectType: dc.ASCObjectType}
+			t.formatInArg = format.AAC_Frame_In{ObjectType: dc.ASCObjectType}
 		}
 	case "stts":
 		d.FieldU8("version")
@@ -853,13 +853,18 @@ func decodeBox(ctx *decodeContext, d *decode.D, typ string) {
 			}
 		})
 	case "ctts":
-		d.FieldU8("version")
+		version := d.FieldU8("version")
 		d.FieldU24("flags")
 		entryCount := d.FieldU32("entry_count")
 		var i uint64
 		d.FieldStructArrayLoop("entries", "entry", func() bool { return i < entryCount }, func(d *decode.D) {
 			d.FieldS32("sample_count")
-			d.FieldS32("sample_offset")
+			switch version {
+			case 0:
+				d.FieldU32("sample_offset")
+			case 1:
+				d.FieldS32("sample_offset")
+			}
 			i++
 		})
 		// TODO: refactor: merge with stco?
@@ -931,7 +936,7 @@ func decodeBox(ctx *decodeContext, d *decode.D, typ string) {
 		d.FieldU24("flags")
 		d.FieldU32("reserved")
 		if ctx.isParent("covr") {
-			d.FieldFormatOrRawLen("data", d.BitsLeft(), imageFormat, nil)
+			d.FieldFormatOrRawLen("data", d.BitsLeft(), &imageGroup, nil)
 		} else {
 			d.FieldUTF8("data", int(d.BitsLeft()/8))
 		}
@@ -1006,7 +1011,7 @@ func decodeBox(ctx *decodeContext, d *decode.D, typ string) {
 			m = t.moof
 		}
 
-		d.FieldU8("version")
+		version := d.FieldU8("version")
 		sampleCompositionTimeOffsetsPresent := false
 		sampleFlagsPresent := false
 		sampleSizePresent := false
@@ -1054,7 +1059,11 @@ func decodeBox(ctx *decodeContext, d *decode.D, typ string) {
 						d.FieldStruct("sample_flags", decodeSampleFlags)
 					}
 					if sampleCompositionTimeOffsetsPresent {
-						d.FieldU32("sample_composition_time_offset")
+						if version == 0 {
+							d.FieldU32("sample_composition_time_offset")
+						} else {
+							d.FieldS32("sample_composition_time_offset")
+						}
 					}
 				})
 
@@ -1166,27 +1175,54 @@ func decodeBox(ctx *decodeContext, d *decode.D, typ string) {
 			}
 		})
 	case "infe":
-		d.FieldU8("version")
+		version := d.FieldU8("version")
 		d.FieldU24("flags")
-		d.FieldU16("id")
-		d.FieldU16("protection_index")
-		d.FieldUTF8Null("item_name")
-		// TODO: really optional? seems so
-		if d.NotEnd() {
+		if version == 0 || version == 1 {
+			d.FieldU16("item_id")
+			d.FieldU16("item_protection_index")
+			d.FieldUTF8Null("item_name")
 			d.FieldUTF8Null("content_type")
+			if !d.End() {
+				d.FieldUTF8Null("content_encoding")
+			}
 		}
-		if d.NotEnd() {
-			d.FieldUTF8Null("content_encoding")
+		if version == 1 {
+			if !d.End() {
+				d.FieldU32("extension_type")
+			}
+			if !d.End() {
+				d.FieldU32("extension_type_extra")
+			}
+		}
+		if version >= 2 {
+			switch version {
+			case 2:
+				d.FieldU16("item_id")
+			case 3:
+				d.FieldU32("item_id")
+			}
+			d.FieldU16("item_protection_index")
+			itemType := d.FieldUTF8("item_type", 4)
+			d.FieldUTF8Null("item_name")
+			switch itemType {
+			case "mime":
+				d.FieldUTF8Null("content_type")
+				if !d.End() {
+					d.FieldUTF8Null("content_encoding")
+				}
+			case "uri ":
+				d.FieldUTF8Null("item_uri_type")
+			}
 		}
 	case "iinf":
 		version := d.FieldU8("version")
 		d.FieldU24("flags")
 		if version == 0 {
-			_ = d.FieldU16("entry_count")
-			decodeBoxes(ctx, d)
+			d.FieldU16("entry_count")
 		} else {
-			d.FieldRawLen("data", d.BitsLeft())
+			d.FieldU32("entry_count")
 		}
+		decodeBoxes(ctx, d)
 	case "iprp":
 		decodeBoxes(ctx, d)
 	case "ipco":
@@ -1203,7 +1239,7 @@ func decodeBox(ctx *decodeContext, d *decode.D, typ string) {
 			}
 			return s
 		})
-		d.FieldFormat("data", id3v2Format, nil)
+		d.FieldFormat("data", &id3v2Group, nil)
 	case "mehd":
 		version := d.FieldU8("version")
 		d.FieldU24("flags")
@@ -1246,9 +1282,9 @@ func decodeBox(ctx *decodeContext, d *decode.D, typ string) {
 
 		switch {
 		case bytes.Equal(systemID, systemIDWidevine[:]):
-			d.FieldFormatLen("data", int64(dataLen)*8, protoBufWidevineFormat, nil)
+			d.FieldFormatLen("data", int64(dataLen)*8, &protoBufWidevineGroup, nil)
 		case bytes.Equal(systemID, systemIDPlayReady[:]):
-			d.FieldFormatLen("data", int64(dataLen)*8, psshPlayreadyFormat, nil)
+			d.FieldFormatLen("data", int64(dataLen)*8, &psshPlayreadyGroup, nil)
 		case systemID == nil:
 			fallthrough
 		default:
@@ -1569,7 +1605,7 @@ func decodeBox(ctx *decodeContext, d *decode.D, typ string) {
 				d.FieldU8("color_range")
 			}
 		case "prof":
-			d.FieldFormat("profile", iccProfileFormat, nil)
+			d.FieldFormat("profile", &iccProfileGroup, nil)
 		default:
 			d.FieldRawLen("data", d.BitsLeft())
 		}
@@ -1609,7 +1645,7 @@ func decodeBox(ctx *decodeContext, d *decode.D, typ string) {
 	case "pitm":
 		version := d.FieldU8("version")
 		d.FieldU24("flags")
-		if version < 1 {
+		if version == 0 {
 			d.FieldU16("item_id")
 		} else {
 			d.FieldU32("item_id")
