@@ -17,11 +17,12 @@ import (
 var protobufFS embed.FS
 
 func init() {
-	interp.RegisterFormat(decode.Format{
-		Name:        format.PROTOBUF,
-		Description: "Protobuf",
-		DecodeFn:    protobufDecode,
-	})
+	interp.RegisterFormat(
+		format.Protobuf,
+		&decode.Format{
+			Description: "Protobuf",
+			DecodeFn:    protobufDecode,
+		})
 	interp.RegisterFS(protobufFS)
 }
 
@@ -129,7 +130,7 @@ func protobufDecodeFields(d *decode.D, pbm *format.ProtoBufMessage) {
 }
 
 func protobufDecode(d *decode.D) any {
-	var pbi format.ProtoBufIn
+	var pbi format.Protobuf_In
 	d.ArgAs(&pbi)
 
 	protobufDecodeFields(d, &pbi.Message)
