@@ -9,17 +9,18 @@ import (
 	"github.com/wader/fq/pkg/scalar"
 )
 
-var widevineProtoBufFormat decode.Group
+var widevineProtoBufGroup decode.Group
 
 func init() {
-	interp.RegisterFormat(decode.Format{
-		Name:        format.PROTOBUF_WIDEVINE,
-		Description: "Widevine protobuf",
-		DecodeFn:    widevineDecode,
-		Dependencies: []decode.Dependency{
-			{Names: []string{format.PROTOBUF}, Group: &widevineProtoBufFormat},
-		},
-	})
+	interp.RegisterFormat(
+		format.ProtobufWidevine,
+		&decode.Format{
+			Description: "Widevine protobuf",
+			DecodeFn:    widevineDecode,
+			Dependencies: []decode.Dependency{
+				{Groups: []*decode.Group{format.Protobuf}, Out: &widevineProtoBufGroup},
+			},
+		})
 }
 
 func widevineDecode(d *decode.D) any {
@@ -42,7 +43,7 @@ func widevineDecode(d *decode.D) any {
 		}},
 	}
 
-	d.Format(widevineProtoBufFormat, format.ProtoBufIn{Message: widewinePb})
+	d.Format(&widevineProtoBufGroup, format.Protobuf_In{Message: widewinePb})
 
 	return nil
 }
