@@ -45,15 +45,15 @@ type DecodeValue interface {
 }
 
 func (i *Interp) _registry(c any) any {
-	uniqueFormats := map[string]decode.Format{}
+	uniqueFormats := map[string]*decode.Format{}
 
 	groups := map[string]any{}
 	formats := map[string]any{}
 
-	for groupName := range i.Registry.FormatGroups {
+	for _, g := range i.Registry.Groups() {
 		var group []any
 
-		for _, f := range i.Registry.MustFormatGroup(groupName) {
+		for _, f := range g.Formats {
 			group = append(group, f.Name)
 			if _, ok := uniqueFormats[f.Name]; ok {
 				continue
@@ -61,7 +61,7 @@ func (i *Interp) _registry(c any) any {
 			uniqueFormats[f.Name] = f
 		}
 
-		groups[groupName] = group
+		groups[g.Name] = group
 	}
 
 	for _, f := range uniqueFormats {
@@ -77,8 +77,8 @@ func (i *Interp) _registry(c any) any {
 		var dependenciesVs []any
 		for _, d := range f.Dependencies {
 			var dNamesVs []any
-			for _, n := range d.Names {
-				dNamesVs = append(dNamesVs, n)
+			for _, g := range d.Groups {
+				dNamesVs = append(dNamesVs, g.Name)
 			}
 			dependenciesVs = append(dependenciesVs, dNamesVs)
 		}
@@ -86,8 +86,8 @@ func (i *Interp) _registry(c any) any {
 			vf["dependencies"] = dependenciesVs
 		}
 		var groupsVs []any
-		for _, n := range f.Groups {
-			groupsVs = append(groupsVs, n)
+		for _, g := range f.Groups {
+			groupsVs = append(groupsVs, g.Name)
 		}
 		if len(groupsVs) > 0 {
 			vf["groups"] = groupsVs
@@ -163,11 +163,17 @@ func (i *Interp) _registry(c any) any {
 	}
 }
 
-func (i *Interp) _toValue(c any, opts map[string]any) any {
-	return toValue(
-		func() Options { return OptionsFromValue(opts) },
-		c,
-	)
+func (i *Interp) _toValue(c any, om map[string]any) any {
+	opts, err := OptionsFromValue(om)
+	if err != nil {
+		return err
+	}
+
+	v, err := toValue(func() (*Options, error) { return opts, nil }, c)
+	if err != nil {
+		return err
+	}
+	return v
 }
 
 type decodeOpts struct {
@@ -181,7 +187,7 @@ func (i *Interp) _decode(c any, format string, opts decodeOpts) any {
 
 	// TODO: progress hack
 	// would be nice to move all progress code into decode but it might be
-	// tricky to keep track of absolute positions in the underlaying readers
+	// tricky to keep track of absolute positions in the underlying readers
 	// when it uses BitBuf slices, maybe only in Pos()?
 	if bbf, ok := c.(*openFile); ok {
 		filename = bbf.filename
@@ -229,19 +235,19 @@ func (i *Interp) _decode(c any, format string, opts decodeOpts) any {
 	if err != nil {
 		return err
 	}
-	decodeFormat, err := i.Registry.FormatGroup(formatName)
+	decodeGroup, err := i.Registry.Group(formatName)
 	if err != nil {
 		return err
 	}
 
-	dv, formatOut, err := decode.Decode(i.EvalInstance.Ctx, bv.br, decodeFormat,
+	dv, formatOut, err := decode.Decode(i.EvalInstance.Ctx, bv.br, decodeGroup,
 		decode.Options{
 			IsRoot:      true,
 			FillGaps:    true,
 			Force:       opts.Force,
 			Range:       bv.r,
 			Description: filename,
-			FormatInArgFn: func(init any) any {
+			ParseOptsFn: func(init any) any {
 				v, err := copystructure.Copy(init)
 				if err != nil {
 					return nil
@@ -305,19 +311,22 @@ func valueHas(key any, a func(name string) any, b func(key any) any) any {
 	return b(key)
 }
 
-// optsFn is a function as toValue is used by tovalue/0 so needs to be fast
-func toValue(optsFn func() Options, v any) any {
-	switch v := v.(type) {
-	case JQValueEx:
-		if optsFn == nil {
-			return v.JQValueToGoJQ()
+// TODO: make more efficient somehow? shallow values but might be hard
+// when things like tovalue.key should behave like a jq value and not a decode value etc
+func toValue(optsFn func() (*Options, error), v any) (any, error) {
+	return gojqex.ToGoJQValueFn(v, func(v any) (any, error) {
+		switch v := v.(type) {
+		case JQValueEx:
+			if optsFn == nil {
+				return v.JQValueToGoJQ(), nil
+			}
+			return v.JQValueToGoJQEx(optsFn), nil
+		case gojq.JQValue:
+			return v.JQValueToGoJQ(), nil
+		default:
+			return v, nil
 		}
-		return v.JQValueToGoJQEx(optsFn)
-	case gojq.JQValue:
-		return v.JQValueToGoJQ()
-	default:
-		return v
-	}
+	})
 }
 
 type decodeValueKind int
@@ -427,10 +436,7 @@ func makeDecodeValueOut(dv *decode.Value, kind decodeValueKind, out any) any {
 				decodeValueBase: decodeValueBase{dv: dv},
 			}
 		case Binary:
-			return decodeValue{
-				JQValue:         vvv,
-				decodeValueBase: decodeValueBase{dv: dv},
-			}
+			return vvv
 
 		default:
 			panic(fmt.Sprintf("unreachable vv %#+v", vvv))
@@ -449,7 +455,7 @@ func (dvb decodeValueBase) DecodeValue() *decode.Value {
 	return dvb.dv
 }
 
-func (dvb decodeValueBase) Display(w io.Writer, opts Options) error { return dump(dvb.dv, w, opts) }
+func (dvb decodeValueBase) Display(w io.Writer, opts *Options) error { return dump(dvb.dv, w, opts) }
 func (dvb decodeValueBase) ToBinary() (Binary, error) {
 	return Binary{br: dvb.dv.RootReader, r: dvb.dv.InnerRange(), unit: 8}, nil
 }
@@ -536,7 +542,7 @@ func (dvb decodeValueBase) JQValueKey(name string) any {
 	case "_gap":
 		switch vv := dv.V.(type) {
 		case Scalarable:
-			return vv.ScalarGap()
+			return vv.ScalarIsGap()
 		default:
 			return false
 		}
@@ -602,7 +608,7 @@ func (v decodeValue) JQValueKey(name string) any {
 func (v decodeValue) JQValueHas(key any) any {
 	return valueHas(key, v.decodeValueBase.JQValueKey, v.JQValue.JQValueHas)
 }
-func (v decodeValue) JQValueToGoJQEx(optsFn func() Options) any {
+func (v decodeValue) JQValueToGoJQEx(optsFn func() (*Options, error)) any {
 	if !v.bitsFormat {
 		return v.JQValueToGoJQ()
 	}
@@ -621,7 +627,12 @@ func (v decodeValue) JQValueToGoJQEx(optsFn func() Options) any {
 		return err
 	}
 
-	s, err := optsFn().BitsFormatFn(brC)
+	opts, err := optsFn()
+	if err != nil {
+		return err
+	}
+
+	s, err := opts.BitsFormatFn(brC)
 	if err != nil {
 		return err
 	}
@@ -691,13 +702,29 @@ func (v ArrayDecodeValue) JQValueHas(key any) any {
 			return intKey >= 0 && intKey < len(v.Compound.Children)
 		})
 }
-func (v ArrayDecodeValue) JQValueToGoJQ() any {
-	vs := make([]any, len(v.Compound.Children))
-	for i, f := range v.Compound.Children {
-		vs[i] = makeDecodeValue(f, decodeValueValue)
+func (v ArrayDecodeValue) JQValueToGoJQEx(optsFn func() (*Options, error)) any {
+	opts, err := optsFn()
+	if err != nil {
+		return err
+	}
+
+	vs := make([]any, 0, len(v.Compound.Children))
+	for _, f := range v.Compound.Children {
+		switch s := f.V.(type) {
+		case Scalarable:
+			if s.ScalarIsGap() && opts.SkipGaps {
+				// skip, note for arrays this will affect indexes
+				continue
+			}
+		}
+
+		vs = append(vs, makeDecodeValue(f, decodeValueValue))
 	}
 	return vs
 }
+func (v ArrayDecodeValue) JQValueToGoJQ() any {
+	return v.JQValueToGoJQEx(func() (*Options, error) { return &Options{}, nil })
+}
 
 // decode value struct
 
@@ -763,10 +790,25 @@ func (v StructDecodeValue) JQValueHas(key any) any {
 		},
 	)
 }
-func (v StructDecodeValue) JQValueToGoJQ() any {
+func (v StructDecodeValue) JQValueToGoJQEx(optsFn func() (*Options, error)) any {
+	opts, err := optsFn()
+	if err != nil {
+		return err
+	}
+
 	vm := make(map[string]any, len(v.Compound.Children))
 	for _, f := range v.Compound.Children {
+		switch s := f.V.(type) {
+		case Scalarable:
+			if s.ScalarIsGap() && opts.SkipGaps {
+				continue
+			}
+		}
+
 		vm[f.Name] = makeDecodeValue(f, decodeValueValue)
 	}
 	return vm
 }
+func (v StructDecodeValue) JQValueToGoJQ() any {
+	return v.JQValueToGoJQEx(func() (*Options, error) { return &Options{}, nil })
+}
